from enum import Enum
class SearchType(Enum):
    All           =     "All"           
    Comments      =     "Comments"      
    Posts         =     "Posts"         
    Communities   =     "Communities"   
    Users         =     "Users"         
    Url           =     "Url"           

  
  
  
  
  
  