"""
    The jplaw library is designed to allow interactions with the API of Lemmy instances. For function access, see the submodules. For More information on the API, see the Lemmy API documentation.
"""

__version__ = "0.1.4"
"""Current package version"""

__name__ = "jplaw"
"""Package name"""